#include <inttypes.h>

//! LCD width
#define LCD_WIDTH (320)
//! LCD height
#define LCD_HEIGHT (240)
//! Bits per pixel
#define BITS_PER_PIXEL (16)
//! Bytes per pixel
#define BYTES_PER_PIXEL (BITS_PER_PIXEL / 8)

//! Window 0's buffer start address register, buffer 0
#define VIDW00ADD0B0 ((void **)0x4C800064)
//! Window 0's buffer start address register, buffer 1
#define VIDW00ADD0B1 ((void **)0x4C800068)
//! Window 1's buffer start address register
#define VIDW01ADD0 ((void **)0x4C80006C)

//! Window 0's buffer end address register, buffer 0
#define VIDW00ADD1B0 ((void **)0x4C80007C)
//! Window 0's buffer end address register, buffer 1
#define VIDW00ADD1B1 ((void **)0x4C800080)
//! Window 1's buffer end address register
#define VIDW01ADD1 ((void **)0x4C800084)

//! Window 0's buffer size register, buffer 0
#define VIDW00ADD2B0 ((uint32_t *)0x4C800094)
//! Window 0's buffer size register, buffer 1
#define VIDW00ADD2B1 ((uint32_t *)0x4C800098)
//! Window 1's buffer size register
#define VIDW01ADD2 ((uint32_t *)0x4C80009C)

// Just for having non-empty data and BSS sections.
const uint8_t rodata[] = { 0x87, 0x65, 0x43, 0x21 };
uint8_t data[] = { 0x12, 0x34, 0x56, 0x78 };
__attribute__((__section__(".data"))) uint16_t buffer[LCD_WIDTH * LCD_HEIGHT];
uint16_t bss[4];

int _start(void) {
    uint16_t * ptr;

    asm volatile("nop");

    for (uint8_t i = 0; i < sizeof(data) / sizeof(data[0]); i++) {
        data[i] = rodata[i];
        bss[i] = data[i];
    }

    //buffer = (uint16_t *)(*VIDW01ADD0);
    *VIDW01ADD0 = (void *)buffer;
    *VIDW01ADD1 = (void *)((uint8_t *)buffer+sizeof(buffer));
    *VIDW01ADD2 = sizeof(buffer);
    ptr = buffer;
    for (uint32_t i = 0; i < LCD_WIDTH * LCD_HEIGHT / 3; i++) {
        *ptr++ = 0x7C00; // Full red, no transparency.
    }
    for (uint32_t i = 0; i < LCD_WIDTH * LCD_HEIGHT / 3; i++) {
        *ptr++ = 0x03E0; // Full green, no transparency.
    }
    for (uint32_t i = 0; i < LCD_WIDTH * LCD_HEIGHT / 3; i++) {
        *ptr++ = 0x001F; // Full blue, no transparency.
    }

    asm volatile("nop");
    asm volatile("0: b 0b");
    asm volatile("nop");
    __builtin_unreachable();
}
